package gg.malloc.defense;

import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.World;

import java.util.logging.Logger;
import java.util.logging.Level;

import java.util.ArrayList;
import java.util.HashMap;

import gg.malloc.defense.model.Arena;
import gg.malloc.defense.model.Spawnpoint;
import gg.malloc.defense.model.Game;

import gg.malloc.defense.games.LinearGame;
import gg.malloc.defense.games.ScaledWaves;

import gg.malloc.defense.engine.GameRunner;

import gg.malloc.defense.commands.AddPlayerCommand;
import gg.malloc.defense.commands.SetStageCommand;

public class Plugin extends JavaPlugin {
  ArrayList<Arena> m_arenas = new ArrayList<>();
  ArrayList<Game> m_games = new ArrayList<>();
  HashMap<World, GameRunner> m_runningGames = new HashMap<>();

  private class TestSpawn implements Spawnpoint {
    Location m_location;

    public TestSpawn(Location location) {
      m_location = location;
    }

    @Override
    public Location getLocation() {
      return m_location;
    }

    @Override
    public String getName() {
      return "Mob Spawner";
    }

    @Override
    public String getID() {
      return "mob-spawner";
    }
  }

  public void debuginfo() {
    getLogger().info("Debug Info:");
  }

  @Override
  public void onEnable() {
    getLogger().info("Malloc Defense registered");
    getLogger().setLevel(Level.FINEST);
    setupDemoGame();
    getCommand("setstage").setExecutor(new SetStageCommand(this));
    getCommand("addplayer").setExecutor(new AddPlayerCommand(this));
    getCommand("debuginfo").setExecutor(new DebuginfoCommand(this));
  }

  public GameRunner getRunnerForWorld(World world) {
    GameRunner ret;
    if (m_runningGames.containsKey(world)) {
      ret = m_runningGames.get(world);
    } else {
      ret = new GameRunner(this, m_games.get(0), m_arenas.get(0));
      m_runningGames.put(world, ret);
      getServer().getPluginManager().registerEvents(new GameEventHandler(ret), this);
    }
    return ret;
  }

  void setupDemoGame() {
    getLogger().info("Setting up demo data'");
    World testWorld = getServer().getWorld("world");
    Spawnpoint[] spawnpoints = new Spawnpoint[4];
    Location spawnCenter = testWorld.getSpawnLocation();
    for(int i = 0; i < 4; i++) {
      Location spawnLocation = spawnCenter.add(Math.random() * 3, 0, Math.random() * 3);
      spawnpoints[i] = new TestSpawn(spawnLocation);
    }
    m_arenas.add(new MemoryArena("Test Arena", testWorld, spawnpoints));
    m_games.add(new ScaledWaves());
  }
}
